/****************************************************************************/
/*   FILE JMODEM_C.C                                                        */
/*   Created 11-JAN-1990                   Richard B. Johnson               */
/*                                         405 Broughton Drive              */
/*                                         Beverly, Massachusetts 01915     */
/*                                         BBS (508) 922-3166               */
/*                                                                          */
/*   File I/O                                                               */
/*   file_io();                                                             */
/*   All of the file I/O is called from this one block to ease portability  */
/*                                                                          */
/****************************************************************************/
#include <stdio.h>                       /* Used for NULL                   */
#include <dos.h>                         /* Used for file-size              */
#include <io.h>                          /* Ysed for Unix / DOS type files  */
#include <fcntl.h>                       /* Used for O_BINARY, etc          */
#include <string.h>                      /* Used for _strchr(), etc         */
#include "jmodem.h"                      /* Used for JMODEM primatives      */
/****************************************************************************/
/*   Note:                                                                  */
/*   I originally used fopen();, fclose();, fread();, and fwrite(); the     */
/*   default C stream-I/O files. However, a bug in Microsoft's implementa-  */
/*   tion results in corruption of files larger than about 512k. The bug    */
/*   inserts a null into the  files every approximately 512k.               */
/*   Therefore, I changed to the UNIX-style disk I/O.                       */
/*                                                                          */
/****************************************************************************/
/* Merged from \include\sys\stat.h because improperly organized header-file */
/* REQUIRES that two other files be included first! Thanks, Microsoft...    */
/* Of course if one is using TurboC then this is no prob! Thanks Borland... */
#if defined (TURBOC)
    #include <sys\stat.h>
    #include <dir.h>                    /* struct for ffblk                 */
    #define find_t ffblk
    #define _A_NORMAL 0
#else
#define S_IFMT      0170000             /* file type mask                   */
#define S_IFDIR     0040000             /* directory                        */
#define S_IFCHR     0020000             /* character special                */
#define S_IFREG     0100000             /* regular                          */
#define S_IREAD     0000400             /* read permission, owner           */
#define S_IWRITE    0000200             /* write permission, owner          */
#define S_IEXEC     0000100             /* execute/search permission, owner */
#endif
unsigned char old[]=".OLD";             /* For renaming files               */

/****************************************************************************/
unsigned short file_io(command, handle, buffer, len)
unsigned short command;                           /* Read/write/open, etc.  */
register short *handle;                           /* File handle            */
register unsigned char *buffer;                   /* Working buffer pointer */
unsigned short len;                               /* Bytes to read/write    */
{
    struct find_t o_file;                         /* To get file size       */
    unsigned char temp[65];                       /* To rename a file       */
    unsigned char *dot;                           /* Find the "."           */
    switch (command)                              /* GOTO in disguise       */
    {
/****************************************************************************/
/*      Open the file for read. If the open fails, return non-zero.         */
/****************************************************************************/
        case OPEN_READ:
        {
            syst.s_txt = buffer;
            screen (SCR_FIL,&syst);
            if ( (*handle = open(buffer,O_RDONLY|O_BINARY)) != -1 )
            {
#if defined (TURBOC)
                findfirst(buffer,&o_file,_A_NORMAL);
                syst.s_byt = o_file.ff_fsize;
#else
                _dos_findfirst(buffer,_A_NORMAL, &o_file);
                syst.s_byt = o_file.size;         /* Show file size        */
#endif
                screen (SCR_FOK,&syst);           /* Show success          */
                return JM_NRM;
            }
        screen (SCR_FNF,NULL);                    /* Show failure          */
        return JM_FNF;
        }
/****************************************************************************/
/*         Create a new file. If the file already exists, rename it.        */
/****************************************************************************/
       case CREATE:
        {
            syst.s_txt = buffer;
            screen (SCR_FIL,&syst);
            if ( (*handle = open(buffer,O_RDONLY|O_BINARY)) != -1 )
            {                                     /* If file already exists */
                close (*handle);                  /* Close the open file    */
                strcpy (temp,buffer);             /* Copy the file name     */
                dot = strchr (temp, '.');         /* Find "." in string     */
                if (dot)                          /* If there was a dot     */
                    strcpy (dot, old);            /* Substitute ".OLD"      */
                else
                    strcat (temp, old);           /* Append "filename.OLD"  */
                syst.s_txt = temp;
                if ( rename(buffer,temp) )        /* Try to rename the file */
                {
                    screen (SCR_FCR,&syst);       /* Tell user can't rename */
                    return JM_REN;                /* Quit                   */
                }
                screen (SCR_FRN,&syst);           /* Tell user we renamed   */
            }
/* Note:                                                                    */
/*     Microsoft's own rules of MS-DOS state that a file being CREATED is   */
/*     open for WRITE (naturally, who would wish to READ from an empty      */
/*     file?).                                                              */
/*     Unix also provides this same logic.                                  */
/*     In the following code, if I did not OR in O_RDWR, I would create a   */
/*     file I can't write to!. If I did not also include the [optional]     */
/*     parameters, S_IWRITE OR S_IREAD, I create a file I can't delete!!!   */
/*     These are implementation bugs.                                       */
/*                                                                          */
            if  ( (*handle = open (              /* Open (create) new file  */
                                buffer,          /* File name               */
                                O_CREAT          /* Create new file         */
                               |O_BINARY         /* Binary mode             */
                               |O_RDWR ,         /* Read/write (bug??)      */
                                S_IWRITE         /* Definite bug            */
                               |S_IREAD)) != -1) /* More of same bug        */
            {
                screen (SCR_FOK,NULL);           /* Show success            */
                return JM_NRM;
            }
            screen (SCR_FCR,NULL);               /* Show failure            */
            return JM_CRE;
        }
/****************************************************************************/
/*        Write 'len' bytes to the file. Return the actual bytes written    */
/****************************************************************************/
        case WRITE:
        {
        return ( write ( *handle, buffer, len) );
        }
/****************************************************************************/
/*        Read 'len' bytes from the file. Return the actual bytes read.     */
/****************************************************************************/
        case READ:
        {
        return ( read ( *handle, buffer, len) );
        }
/****************************************************************************/
/*        Close the file. Return any error-code (ignored).                  */
/****************************************************************************/
        case CLOSE:
        {
        return (close (*handle));
        }
/****************************************************************************/
/*        Delete the file named in 'buffer'. Return any error code.         */
/****************************************************************************/
        case DELETE:
        {
        return (remove (buffer));
        }
    }
    return JM_ABT;                                          /* Bad function */
}
/****************************************************************************/
/************************ E N D  O F  M O D U L E  **************************/
